package exnihilocreatio.util;

import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.Setter;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.fml.common.registry.ForgeRegistries;

import javax.annotation.Nonnull;

@AllArgsConstructor
public class ItemInfo {

    public static final ItemInfo EMPTY = new ItemInfo(ItemStack.field_190927_a);

    @Getter
    @Nonnull
    private Item item;
    @Getter
    @Setter
    private int meta;

    public ItemInfo(@Nonnull Item item) {
        this.item = item;
        meta = -1;
    }

    public ItemInfo(@Nonnull ItemStack stack) {
        item = stack.func_77973_b();
        meta = stack.func_77960_j();
    }

    public ItemInfo(@Nonnull Block block, int blockMeta) {
        item = Item.func_150898_a(block);
        meta = block == Blocks.field_150350_a ? -1 : blockMeta;
    }

    public ItemInfo(@Nonnull String string) {
        if (string.isEmpty() || string.length() < 2) {
            item = Items.field_190931_a;
            meta = -1;
            return;
        }
        String[] split = string.split(":");

        Item item = null;
        int meta = -1;

        switch (split.length) {
            case 1:
                item = Item.func_111206_d("minecraft:" + split[0]);
                break;
            case 2:
                try {
                    meta = split[1].equals("*") ? -1 : Integer.parseInt(split[1]);
                    item = Item.func_111206_d("minecraft:" + split[0]);
                } catch (NumberFormatException e) {
                    meta = -1;
                    item = Item.func_111206_d(split[0] + ":" + split[1]);
                }
                break;
            case 3:
                try {
                    meta = split[2].equals("*") ? -1 : Integer.parseInt(split[2]);
                    item = Item.func_111206_d(split[0] + ":" + split[1]);
                } catch (NumberFormatException e) {
                    meta = -1;
                }
                break;
            default:
                item = Items.field_190931_a;
                meta = -1;
        }

        if (item == null){
            this.item = Items.field_190931_a;
            this.meta = -1;
        }
        else {
            this.item = item;
            this.meta = meta;
        }
    }

    public ItemInfo(@Nonnull IBlockState state) {
        item = Item.func_150898_a(state.func_177230_c());
        meta = state.func_177230_c().func_176201_c(state);
    }

    public static ItemInfo getItemInfoFromStack(ItemStack stack) {
        return new ItemInfo(stack);
    }

    public static ItemInfo readFromNBT(NBTTagCompound tag) {
        Item item = ForgeRegistries.ITEMS.getValue(new ResourceLocation(tag.func_74779_i("item")));
        int meta = tag.func_74762_e("meta");

        return item == null ? EMPTY : new ItemInfo(item, meta);
    }

    public String toString() {
        return Item.field_150901_e.func_177774_c(item) + (meta == -1 ? "" : (":" + meta));
    }

    public ItemStack getItemStack() {
        return new ItemStack(item, 1, meta == -1 ? 0 : meta);
    }

    public NBTTagCompound writeToNBT(NBTTagCompound tag) {
        tag.func_74778_a("item", Item.field_150901_e.func_177774_c(item) == null ? "" : Item.field_150901_e.func_177774_c(item).toString());
        tag.func_74768_a("meta", meta);

        return tag;
    }

    public boolean isValid() {
        return item != Items.field_190931_a && meta <= Short.MAX_VALUE;
    }

    public int hashCode() {
        return item.hashCode();
    }

    public boolean equals(Object other) {
        if (other instanceof ItemInfo)
            return ItemStack.func_77989_b(((ItemInfo) other).getItemStack(), getItemStack());
        else if (other instanceof ItemStack)
            return ItemStack.func_77989_b((ItemStack) other, getItemStack());
        else if (other instanceof BlockInfo)
            return ItemStack.func_77989_b(((BlockInfo) other).getItemStack(), getItemStack());

        return false;
    }
}
